{{
This module defines an "interrupt device".

Three registers are used:

 * status is interrupt status register  ISR
 * mask is interrupt mask register      IMR
 * pins is interrupt pin assignment reg IPAR

Usage:
 1 set IMR
 2 set IPAR 
 3 start device
 4 poll ISR

Potentially, the LMM should have interrupt device register
accessor methods (with locks?) to manage the device.
}}


{
'--------------------------------------------------------------
' <save> for testing.
'--------------------------------------------------------------

CON
  _clkmode      = xtal1 + pll16x
  _xinfreq      = 5_000_000

OBJ
  disp: "VGA_Text"
  'disp: "TV_Text"

CON
  base = 16
  'base = 12
    
VAR
  long  status
  long  mask
  long  pins

PUB main | mymask

  disp.start(base)
  
  status := 0
  mymask := 2 '$ff000fff
  mask   := mymask
  pins   := mymask

  start(@status)

  repeat
    if status
      disp.str(string($d,"ISR "))
      disp.hex(status, 8)
      status := 0

'--------------------------------------------------------------
' </save>
'--------------------------------------------------------------
}

VAR
  long cog
  
PUB start(regptr)
  cog := cognew(@pinMonEntry, regptr) + 1

PUB stop
  if cog
    cogstop(cog~ -1)
    
DAT
pinMonEntry   org       0

              mov       statptr, par    
              mov       maskptr, par
              add       maskptr, #4
              mov       pinsptr, par
              add       pinsptr, #8
              mov       invtptr, par
              add       invtptr, #12
              
pinMonLoop    rdlong    IPAR, pinsptr           ' get user pins
              andn      dira, IPAR              ' set input pins

              or        dira, #1                ' for measurement

pinMonLoop1   rdlong    INV,  invtptr           ' get invert register

pinMonLoop2   rdlong    IMR,  maskptr           ' get mask register

pinMonLoop3   mov       IPAR, ina               ' get input state
              xor       IPAR, INV               ' invert sense if required
              and       IPAR, IMR wz
        if_nz wrlong    IPAR, statptr           ' wrlong only if we have an event 
              xor       outa, #1                ' for measurement
              jmp       #pinMonLoop3            ' 250ns with no events (and no xor measurement).

IMR           res 1
IPAR          res 1
INV           res 1

statptr       res 1
maskptr       res 1
pinsptr       res 1
invtptr       res 1

              fit       48