/**
 * @file testint.c
 * Defines an ICCPROP kernel interrupt interface test
 */
#include "vga_text.h"
#include "mouse.h"
#include "interrupt.h"

hwint_t ghwint;

void user_handler(void)
{
    asm("or     dira, #3");
    asm("xor    outa, #3");

    /*
     * printing is usually not the best thing to do at interrupt time.
     */
    if(1)
    {
        vgaText_setXY(0, VGA_TEXT_ROWS-2);
        print("Mouse ");
        vgaText_dec(mouse_present());
        print(" Buttons ");
        vgaText_dec(mouse_buttons());
        print(" X ");
        vgaText_dec(mouse_x());
        print(" Y ");
        vgaText_dec(mouse_y());
        print(" ");
    }
    
    /**
     * If you are using the HWISR feature, you should clear ISR register like this.
     * For some strange reason, using ghwint.isr = 0 crashes the program.
     */
    asm("mov    USER4, #0");
    asm("wrlong USER4, USER3");
    
    /**
     * Enable interrupts before leaving the user_handler
     */
    enable();
}

void main(void)
{
    int id   = 0;
    int count= 0;
    int pins = (1 << MOUSE_PROTO_CLOCK_PIN);
    
    ghwint.isr  = 0;
    ghwint.pins = pins;
    ghwint.imr  = 0;
    ghwint.invt = pins;
       
    id = vgaText_start(16);
    print("Interrupt Test");
    print("\nVGA ");
    vgaText_dec(id);

    id = mouse_start(MOUSE_PROTO_DATA_PIN, MOUSE_PROTO_CLOCK_PIN);
    print("\nMOUSE ");
    vgaText_dec(id);

    id = int_start(&ghwint);

    print("\nINT ");
    vgaText_dec(id);
    print("\n");
    
    ghwint.imr  = pins;
    enable();

    // spin so program doesn't crash.
    while(1)
    {
        disable(); // disable for critical code
        vgaText_setXY(0, VGA_TEXT_ROWS>>1);
        print("\rCounting ");
        vgaText_dec(count++);
        enable();  // reenable for interrupt processing

        if((count % 10) == 0) // give interrupts chance to run
        {
            while(count > 0)
                count--;
        }
    }

}